from dataclasses import dataclass
import requests


@dataclass
class Credentials:
    username: str
    repository_name: str
    token: str

    def get_token(self) -> str:
        return "Bearer " + self.token


@dataclass
class PullRequestData:
    head_branch: str
    base_branch: str
    title: str
    body: str | None


class PullRequest:
    def __init__(self, credentials: Credentials, pr_content: PullRequestData) -> None:
        self.credentials = credentials
        self.pr_content = pr_content

    def get_headers(self) -> dict:
        token = self.credentials.get_token()
        headers = {
            "Accept": "application/vnd.github+json",
            "Authorization": token,
            "Content-Type": "application/json",
        }
        return headers

    def get_body(self) -> dict:
        body = {
            "title": self.pr_content.title,
            "base": self.pr_content.base_branch,
            "head": f"{self.credentials.username}:{self.pr_content.head_branch}",
            "body": self.pr_content.body if self.pr_content.body is not None else "",
        }
        return body

    def get_endpoint_url(self) -> str:
        return f"https://api.github.com/repos/{self.credentials.username}/{self.credentials.repository_name}/pulls"

    def handle_http_response(self, response: requests.Response) -> None:
        match response.status_code:
            case 201:
                print(
                    f"Pull request from '{self.pr_content.head_branch}' to '{self.pr_content.base_branch}' was successfully made"
                )
            case 403:
                print(
                    "You're probably spamming it. Try again later or check if there are changes to make a pull request"
                )
            case _:
                print("Unexpected behavior")
                print(
                    f"STATUS CODE: {response.status_code}\nRESPONSE BODY: {response.json()}"
                )

    async def make_pull_request(self):
        headers = self.get_headers()
        body = self.get_body()
        url = self.get_endpoint_url()
        response = requests.post(url, headers=headers, json=body)
        self.handle_http_response(response)
        exit(1)



import argparse
import json
import asyncio
from pull_request.pullrequest import PullRequest, Credentials, PullRequestData


def get_argument_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Pull requests on command line - Author: Icarus"
    )
    parser.add_argument(
        "-f", "--from", help="Changes are coming from this branch", required=True
    )
    parser.add_argument(
        "-t", "--to", help="Changes are going to that branch", required=True
    )
    parser.add_argument("-T", "--title", help="Pull request title", required=True)
    parser.add_argument("-b", "--body", help="Body of the pull request")
    return parser


def read_credentials() -> Credentials:
    with open("pr.json") as file:
        content = file.read()
        parsed_content = json.loads(content)
        # TODO: add error handling in case of KeyError
        return Credentials(
            parsed_content["username"],
            parsed_content["repository"],
            parsed_content["token"],
        )


async def main():
    argument_parser = get_argument_parser()
    args = vars(argument_parser.parse_args())

    pr_content = PullRequestData(args["from"], args["to"], args["title"], args["body"])
    credentials = read_credentials()
    pr = PullRequest(credentials, pr_content)
    await pr.make_pull_request()

def _main():
    asyncio.run(main())

if __name__ == "__main__":
    _main()



from setuptools import setup

long_description = None
with open("README.md", 'r') as f:
    long_description = f.read()

setup(
   name='pull-requests',
   version='1.1.0',
   description='A command line interface for creating pull requests',
   long_description=long_description,
   long_description_content_type="text/markdown",
   author='Hícaro Dânrlley',
   author_email='hdanrlley1@gmail.com',
   packages=['pull_request'],
   install_requires=['requests'], #external packages as dependencies
   python_requires=">=3.10",
   entry_points={
       "console_scripts": [
           "pull_request = pull_request.main:_main",
       ]
    }
)

